import { ESLintUtils } from "@typescript-eslint/utils";
import { Context } from "./context.js";
import debug from "./debug.js";
const d = debug("rules");
export const createRule = ESLintUtils.RuleCreator((name) => `https://github.com/ciscoheat/eslint-plugin-dci-lint?tab=readme-ov-file#${name}`);
// TODO: Role name checking rule
// TODO: Role splitting config setting
const isContextRegexp = /\W*@DCI-context\b/i;
const _currentContext = [];
const _currentRoleMethod = [];
let _currentFunction = null;
const currentContext = () => _currentContext[_currentContext.length - 1];
const currentRoleMethod = () => _currentRoleMethod[_currentRoleMethod.length - 1];
export const currentFunction = () => _currentFunction;
export const isRoleMethod = (func) => func && currentRoleMethod()?.func === func ? currentRoleMethod() : undefined;
export const isContext = (func) => func && currentContext()?.func === func ? currentContext() : undefined;
export const isInContext = () => currentContext();
export const isInRoleMethod = () => currentRoleMethod();
export const parentContexts = () => _currentContext.slice(0, -1);
const parsedComments = new Set();
const enterFunction = (func, context) => {
    _currentFunction = func;
    const currentCtx = currentContext();
    // Skip if we're already in the same context
    if (currentCtx?.func == func)
        return;
    let roleMethod;
    if (currentCtx) {
        // Check if entering a RoleMethod
        roleMethod = currentCtx.roleMethodFromFunc(func);
        if (roleMethod && currentRoleMethod()?.func != func) {
            d(`Entering RoleMethod ${roleMethod.role}.${roleMethod.method}`);
            _currentRoleMethod.push(roleMethod);
        }
    }
    if (!roleMethod) {
        const source = context.sourceCode;
        const comments = [
            func,
            ...(source.getAncestors ? source.getAncestors(func) : []),
        ]
            .filter((n) => n.parent)
            .flatMap((n) => source.getCommentsBefore(n))
            .filter((c) => !parsedComments.has(c));
        const commentStr = comments.map((c) => c.value).join(" ");
        const hasContextComment = commentStr.match(isContextRegexp);
        comments.forEach((c) => parsedComments.add(c));
        if (hasContextComment) {
            // Too many type checks, disabling linter instead.
            // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment
            const name = 
            // @ts-expect-error Reflecting on type
            // eslint-disable-next-line @typescript-eslint/no-unsafe-member-access
            func.id?.name ?? func.parent?.id?.name ?? func.parent?.key?.name;
            d("Entering Context " + name);
            _currentContext.push(new Context(context, name, func));
        }
    }
};
const exitFunction = (func) => {
    _currentFunction = null;
    const currentCtx = currentContext();
    if (currentCtx?.func == func) {
        d("Exiting Context " + currentCtx.name);
        _currentContext.pop();
    }
    else {
        const roleMethod = currentCtx?.roleMethodFromFunc(func);
        if (roleMethod && currentRoleMethod()?.func == func) {
            d(`Exiting RoleMethod ${roleMethod.role}.${roleMethod.method}`);
            _currentRoleMethod.pop();
        }
    }
};
export const contextRules = (context, rule) => {
    rule[" FunctionDeclaration:exit"] = exitFunction;
    rule[" FunctionExpression:exit"] = exitFunction;
    rule[" ArrowFunctionExpression:exit"] = exitFunction;
    rule[" FunctionDeclaration"] = (func) => enterFunction(func, context);
    rule[" FunctionExpression"] = (func) => enterFunction(func, context);
    rule[" ArrowFunctionExpression"] = (func) => enterFunction(func, context);
    return rule;
};
//# sourceMappingURL=DCIRuleHelpers.js.map